/*------------------------------------------------------------------

   VFO System for ESP32-DevKitC  Ver 1.00

        by T.Uebo / JF3HZB

          Feb. 10, 2019


Band limits and switching works!
Based on VFOsys_astep_20201013
 20201027
USB/LSB switching works.
USB/LSB display works.


--------------------------------------------------------------------*/

/*------Hard ware Configuration ---------------------
<<ESP32-DevKitC>>
pin No.  Connection
  17 :  Rotary Encoder A
  16 :  Rotary Encoder B

  22 :  si5351A SCL
  21 :  si5351A SDA

  18 :  SCK    / ST7735,SEPS525(128x160 display)  
  23 :  MOSI   / ST7735,SEPS525(128x160 display)
   5 :  CS     / ST7735,SEPS525(128x160 display)  
   2 :  DC(A0) / ST7735,SEPS525(128x160 display)
  15 :  RESET  / ST7735,SEPS525(128x160 display)


<<si5351A>>
CLK0 : Car Signal (I)
CLK1 : Car Signal (Q)
CLK2 : Lo Signal
------------------------------------------------*/

/*-------------------------------------------------------
   Frequency settings
--------------------------------------------------------*/

#define LSB       1647000                   // carrier mixer for LSB
#define USB       1650000                   // carrier mixer for USB

#define init_freq 3630000      // Initial Frequncy[Hz]
//int32_t offset_frq = 8998500;  // Offset Frequency[Hz]
int32_t offset_frq = LSB;  // Offset Frequency[Hz]
//int32_t car_frq = 8998500;     // Carrier Frequncy[Hz]
int32_t car_frq = LSB;     // Carrier Frequncy[Hz]
unsigned char f_carON = 1;     // ON/OFF Car signal

#define freq_step 10           // Min step[Hz]

/*----------------------------------------------------------
   Adaptive step control parameters
-----------------------------------------------------------*/
uint32_t vth=2;   	  // Velocity threshold for acceleration
float Racc=0.002;     // Rate of acceleration
float Rdec=1.0;       // Rate of deceleration
float MaxL=1500.0;	  // Max Step[Hz] = freq_step + Racc*MaxL*MaxL

/*----------------------------------------------------------------------------------
    Control flags etc
-----------------------------------------------------------------------------------*/
uint8_t f_fchange;  // if frequency changed, set this flag to 1
uint8_t f_cchange;  // if Car frequency and/or "f_carON" changed, set this flag to 1
uint8_t f_dchange;  // if need to renew display, set this flag to 1

int last_sideband = 0;

byte updateFlag=HIGH;

// synthesiser default settings on start up.
unsigned long genfrequency  =   3630000;      // frequency of signal used to calc VFO
unsigned long frequency     =   3630000;      // fre uency temp calcs

static signed char ham_band_index = 1;    // start on 80M band

unsigned long frequency0 =  1810000;          // band0 frequency
unsigned long frequency1 =  3630000;          // band1 frequency
unsigned long frequency2 =  7150000;          // band2 frequency
unsigned long frequency3 = 10100000;          // band3 frequency
unsigned long frequency4 = 14100000;          // band4 frequency
unsigned long frequency5 = 18068000;          // band5 frequency
unsigned long frequency6 = 21000000;          // band6 frequency
unsigned long frequency7 = 24890000;          // band7 frequency
unsigned long frequency8 = 28000000;          // band8 frequency

/*--------------------------------------------------------
   pin assign
----------------------------------------------------------*/

#define PULSE_INPUT_PIN 17 // Rotaty Encoder A was 34
#define PULSE_CTRL_PIN 16  // Rotaty Encoder B was 35

const int tone_pin = 14;
// keypad matrix

const int Row0 = 33;
const int Row1 = 34;      // input only
const int Row2 = 35;      // input only


const int Col0 = 26;
const int Col1 = 19;
const int Col2 = 14;

int repeat=LOW;             // autorepaet flag

byte key = 0;


//------------------------------------------------------------------------------
#define NAME "VFO System"
#define VERSION "Ver.20201027PM"
#define ID "by JF3HZB mod DCP"


#include "driver/pcnt.h"
#include "display.h"
#include "graph.h"
#include "dial.h"
#include "si5351.h"

/*-----------------------------------------------------------------------------
 *       Global
-----------------------------------------------------------------------------*/
float L=0.0;
long afstp;
long frq=init_freq;
int16_t RE_Count = 0;
uint8_t f_redraw; 
extern char f_rev;
extern uint32_t cl_BG;


/*--------------------------------------------------------------------------
        Timer ISR
---------------------------------------------------------------------------*/
hw_timer_t * timer = NULL;
void IRAM_ATTR onTimer(){}


void get_button()
{   key=0;
     digitalWrite(Col0,HIGH) ;
     delay(1);
   if (digitalRead(Row0) == HIGH) key=1;
   if (digitalRead(Row1) == HIGH) key=2;
   if (digitalRead(Row2) == HIGH) key=3; 
   digitalWrite(Col0,LOW);
   digitalWrite(Col1,HIGH);
        delay(1);
   if (digitalRead(Row0) == HIGH) key=4;
   if (digitalRead(Row1) == HIGH) key=5;
   if (digitalRead(Row2) == HIGH) key=6; 
   digitalWrite(Col1,LOW);
   digitalWrite(Col2,HIGH);
        delay(1);
   if (digitalRead(Row0) == HIGH) key=7;
   if (digitalRead(Row1) == HIGH) key=8;
   if (digitalRead(Row2) == HIGH) key=9; 
   digitalWrite(Col2,LOW);

}

 void read_buttons()
 {
    repeat=LOW;                 // autorepaet flag
    updateFlag=LOW;             // pre-lear the flag.
    get_button();
   if(key !=0)
   { updateFlag=HIGH; 
   
   }   

 } 


//-----------------------------------------------------------------------------------------------
void setup() {
//-----------------------------------------------------------------------------------------------
    char str[64];
    //--------- create tasks on core0 --------------------------------
    xTaskCreatePinnedToCore(task0, "Task0", 4096, NULL, 1, NULL, 0);
        
    //--------- Set up Interrupt Timer -------------------------------
    timer = timerBegin(0, 80, true); //use Timer0, div80 for 1us clock
    timerAttachInterrupt(timer, &onTimer, true);
    timerAlarmWrite(timer, 10000, true); // T=10000us
    timerAlarmEnable(timer); // Start Timer
    
    //--- Counter setup for Rotary Encoder ---------------------
    pcnt_config_t pcnt_config_A;// structure for A   
    pcnt_config_t pcnt_config_B;// structure for B
    //
    pcnt_config_A.pulse_gpio_num = PULSE_INPUT_PIN;
    pcnt_config_A.ctrl_gpio_num = PULSE_CTRL_PIN;
    pcnt_config_A.lctrl_mode = PCNT_MODE_REVERSE;
    pcnt_config_A.hctrl_mode = PCNT_MODE_KEEP;
    pcnt_config_A.channel = PCNT_CHANNEL_0;
    pcnt_config_A.unit = PCNT_UNIT_0;
    pcnt_config_A.pos_mode = PCNT_COUNT_INC;
    pcnt_config_A.neg_mode = PCNT_COUNT_DEC;
    pcnt_config_A.counter_h_lim = 10000;
    pcnt_config_A.counter_l_lim = -10000;
    //
    pcnt_config_B.pulse_gpio_num = PULSE_CTRL_PIN;
    pcnt_config_B.ctrl_gpio_num = PULSE_INPUT_PIN;
    pcnt_config_B.lctrl_mode = PCNT_MODE_KEEP;
    pcnt_config_B.hctrl_mode = PCNT_MODE_REVERSE;
    pcnt_config_B.channel = PCNT_CHANNEL_1;
    pcnt_config_B.unit = PCNT_UNIT_0;
    pcnt_config_B.pos_mode = PCNT_COUNT_INC;
    pcnt_config_B.neg_mode = PCNT_COUNT_DEC;
    pcnt_config_B.counter_h_lim = 10000;
    pcnt_config_B.counter_l_lim = -10000;
    //
    pcnt_unit_config(&pcnt_config_A);//Initialize A
    pcnt_unit_config(&pcnt_config_B);//Initialize B
    pcnt_counter_pause(PCNT_UNIT_0);
    pcnt_counter_clear(PCNT_UNIT_0);
    pcnt_counter_resume(PCNT_UNIT_0); //Start

  
    display_init();    
    GRAM_clr();  
    sprintf(str,  NAME  ); disp_str16(str,20, 90, 0x00ffff);
    sprintf(str, VERSION); disp_str12(str,36, 50, 0x00ffff);   
    sprintf(str,    ID  ); disp_str8(str,50, 20, 0x00ffff);
    trans65k();
    f_redraw=1;

   Serial.begin(115200);
    
   pinMode(Row0,INPUT);
   pinMode(Row1,INPUT);
   pinMode(Row2,INPUT);
 
   pinMode(Col0,OUTPUT);
   pinMode(Col1,OUTPUT);
   pinMode(Col2,OUTPUT);
   digitalWrite(Col0,LOW);
   digitalWrite(Col1,LOW);
   digitalWrite(Col2,LOW);
    
   pinMode (tone_pin,OUTPUT);
   
   Serial.println("Starting Up");

    delay(1000); 
    init_Dial();
    GRAM_clr();

    si5351_init();
         
    f_fchange = 1;
    f_cchange = 1;
    f_dchange = 1;
    f_redraw=0;    
}


  //-----------------------------------------------------------------------------------------------
void loop() {    // (core1)
//-----------------------------------------------------------------------------------------------

  
    char str[64];
   
    if(f_dchange==1){
        f_dchange=0;        
        //GRAM_clr();
        boxfill(0,0,Nx-1,Ny-1,cl_BG);     
        //Display Dial
        Dial(frq);
         
        box(7,100,153,126, 0xa0a0a0);
        box(6,99,154,127, 0xa0a0a0);
  
        //-------- Display Digital Frquency ---------------------------------------
        sprintf(str, "%3d.%03d,%02d",  frq/1000000, (frq/1000)%1000, (frq/10)%100 );
        disp_str16(str,17, 105, 0xffd080);  
//        sprintf(str, "MHz" );
        if  (last_sideband == 1)
    {            
        sprintf(str, "USB" );
    }
    else
    {   sprintf(str, "LSB" );
    }
        disp_str12(str,120, 106, 0xffd080);
          
        if(f_redraw==0){
          trans65k();
          f_redraw=1;
        }            
    }
    read_buttons();
    

 if (key==3)
   {  f_dchange=1;
      set_band(1);
   }
 if (key==6)
    {    f_dchange=1;
      set_band(-1); 
    }

 if (key==1)
    {car_frq = USB;     // USB
    last_sideband = 1;
    f_cchange=1;
    f_dchange=1;
    }

 if (key==4)
    {car_frq = LSB;     // LSB
    last_sideband = 0;
    f_cchange=1;
    f_dchange=1;
    }
 
        while ((key!=0) &(repeat==LOW))  // wait untill button is released....
     {  get_button();}
    
}


//-----------------------------------------------------------------------------------------------
void loopWorks() {    // (core1)
//-----------------------------------------------------------------------------------------------
    char str[64];
   
    if(f_dchange==1){
        f_dchange=0;    
           
        //GRAM_clr();
        boxfill(0,0,Nx-1,Ny-1,cl_BG); 		
        //Display Dial
        Dial(frq);
         
        box(7,100,153,126, 0xa0a0a0);
        box(6,99,154,127, 0xa0a0a0);
  
        //-------- Display Digital Frquency ---------------------------------------
        sprintf(str, "%3d.%03d,%02d",  frq/1000000, (frq/1000)%1000, (frq/10)%100 );
        disp_str16(str,17, 105, 0xffd080);              
        sprintf(str, "MHz" );
        disp_str12(str,120, 106, 0xffd080);
          
        if(f_redraw==0){
          trans65k();
          f_redraw=1;
        }            
    }
}


/*-----------------------------------------------------------------------------------------------
        Alternative Loop (core0)
------------------------------------------------------------------------------------------------*/
void task0(void* arg)
{
     while (1)
     {    
         pcnt_get_counter_value(PCNT_UNIT_0, &RE_Count);          
         int count=RE_Count;
         pcnt_counter_clear(PCNT_UNIT_0);
         
         if(f_rev==1) count=-count;
                  
        
        if(count!=0){                           
          afstp=(long)(0.5 + (float)(count)*((float)freq_step + Racc*L*L) );         
          if(abs(count)>=vth) L+=1.0*(float)( abs(count) - vth );
          if(L>MaxL) L=MaxL;                                  
        }else{         
          L-=Rdec;
          if(L<0) L=0;
        }
 
if (updateFlag=HIGH)  {


  switch (ham_band_index) {       // limit to SSB part of bands
    case 0:     // 160 Meters
      frq = constrain(frq, 1800000, 1875000);
      frequency0 = frq;
      genfrequency=(frq);
      break;
   
    case 1:     // 80 Meters
      frq = constrain(frq, 3500000, 3800000);
      frequency1 = frq;
      genfrequency=(frq);
      break;

    case 2:   // 40 Meters
      frq = constrain(frq, 7000000, 7300000);
      frequency2 = frq;
      genfrequency=frq;
      break;

    case 3:     // 30 Meters
      frq = constrain(frq, 10100000, 10150000);
      frequency3 = frq;
      genfrequency=(frq);
      break;
   
    case 4:   // 20 Meters
      frq = constrain(frq, 14100000, 14350000);
      frequency4 = frq;
      genfrequency=frq;
      break;

    case 5:     // 17 Meters
      frq = constrain(frq, 18068000, 18168000);
      frequency5 = frq;
      genfrequency=(frq);
      break;
   
    case 6:   // 15 Meters
      frq = constrain(frq, 21000000, 21450000);
      frequency6 = frq;
      genfrequency=frq;
      break;

    case 7:     // 12 Meters
      frq = constrain(frq, 24890000, 24990000);
      frequency7 = frq;
      genfrequency=(frq);
      break;
   
    case 8:   // 10 meters
      frq = constrain(frq, 28000000, 29700000);
      frequency8 = frq;
      genfrequency=frq;
      break;
  }
    updateFlag=LOW; 
}

    //---Update frequency -------------------------------
        if(afstp!=0){
          frq+=afstp;

          afstp=0;
          f_dchange=1;
          f_fchange=1;                                                     
        }

         //-------------------------------------
         if(f_fchange==1){
            f_fchange=0;
            // Output Lo freq
            set_freq( frq + offset_frq );                
         }
         
         //-------------------------------------
         if(f_cchange==1){
            f_cchange=0;
            // Output Car freq            
            set_car_freq(car_frq, f_carON, 0);                         
         }

         if(f_redraw==1){
            Transfer_Image();
            f_redraw=0;
         }
                   
         delay(1);
     }
}

void set_band(int inc) {   
    unsigned long ham_band[] = { frequency0,frequency1,frequency2,frequency3,frequency4,frequency5,frequency6,frequency7,frequency8 };

  ham_band[ham_band_index]=frq;     // save the old setting
  ham_band_index = (ham_band_index+inc);
  ham_band_index = constrain(ham_band_index, 0, 8); 

  frq = ham_band[ham_band_index];     // change frequency band
      f_fchange = 1;
  
}
